/* 
*   scrnopt: screen update optimizer
*   91/10/03 john h. dubois iii (jhdiii@armory.com)
*   92/09/14 Use unsigned chars so that high chars won't be sign extended
*            into goofy attributes.  
*            Avoid resetting terminal colors.
*/

#include <curses.h>
#include <signal.h>

char NewscreenChar = '\014';
WINDOW *NextScreen;
char *What = "@(#) screenopt 1.0 91/10/03";

#define BUFSIZE 10240

void Display(chtype C);
void PrintBuf(unsigned char *Buf,int Num);
void Cleanup(int foo);

main(argc,argv)
int argc;
char *argv[];
{
    char *TType;
    char c;
    extern char *optarg;
    unsigned char Buffer[BUFSIZE];
    int Err = 0;
    int Num;

    while ((c = getopt(argc,argv,"t:n:h")) != EOF)
	switch (c) {
	case 't':
	    if (!(TType = (char *) malloc(strlen(optarg) + 6))) {
		puts("malloc failed.");
		exit(1);
	    }
	    putenv(strcat(strcpy(TType,"TERM="),optarg));
	    putenv("TERMCAP=");
	    break;
	case 'h':
	    puts("screenopt: optimize screen updates.\n"\
"screenopt optimizes screen updates in applications where a large amount of\n"\
"similar data is repeatedly displayed.  Lines written to screenopt are\n"\
"saved in a buffer.  When a newscreen character is received, the buffer is\n"\
"displayed on the terminal and then saved for comparison to the next screen.\n"\
"When the buffer is displayed, it is compared to the previous buffer\n"\
"and only those portions of the screen that have changed are updated.\n"\
"If the first character after the newscreen character is a newline,\n"\
"it is discarded.  Only as many lines as the terminal can display are\n"\
"saved in the buffer.\n"\
"Options:\n"\
"The environment variable TERM, used to determine how to do the optimized\n"\
"screen updates, can be overridden with the option -t termtype\n"\
"The newscreen character can be set with -n c\n"\
"It is ^L (formfeed, control-L) by default.\n"\
"-h gives this help screen.\n");
	    exit(0);
	case 'n':
	    NewscreenChar = *optarg;
	    break;
	case '?':
	    Err = 1;
	}
    if (Err) {
	puts("Use -h for help.");
	exit(1);
    }
    signal(SIGINT,Cleanup);
    signal(SIGQUIT,Cleanup);
    initscr();
    refresh();
    NextScreen = newwin(LINES,COLS,0,0);
    noraw();
    nocbreak();
    while ((Num = read(0,Buffer,BUFSIZE)) > 0)
	PrintBuf(Buffer,Num);
    Cleanup(0);
}

void Cleanup(int foo)
{
    /* Use this instead of endwin() because endwin() restores 
       "original" terminal color pair which may not be desirable, 
       and we're not changing color in this program anyway. */
    move(LINES - 1,0);
    refresh();
    reset_shell_mode();
    exit(0);
}

void PrintBuf(Buf,Num)
unsigned char *Buf;
int Num;
{
    unsigned char *End = Buf + Num;

    while (Buf != End)
	Display(*(Buf++));
}

/*
 * Doing a clear forces a terminal screen clear on next refresh,
 * so write to a separate window and then overlay it on stdscr
 */
void Display(chtype C)
{
    static int New = 1;
    int Row, Col;
    static int Printing = 1;

    if (C == NewscreenChar) {
	overwrite(NextScreen,stdscr);
	refresh();
	wclear(NextScreen);
	New = 1;
	Printing = 1;
    }
    else if (Printing)
	if (C == '\n')
	    if (New)
		New = 0;
	    else {
		getyx(NextScreen,Row,Col);
		if (Row == (LINES - 1))
		    Printing = 0;
		else
		    wprintw(NextScreen,"\n");
	    }
	else {
	    waddch(NextScreen,C);
	    New = 0;
	}
}
